/*** 
 * @Author: Chuanbin Wang
 * @Date: 1970-01-01 08:00:00
 * @LastEditTime: 2021-08-27 11:05:46
 * @LastEditors: Chuanbin Wang
 * @FilePath: /libuniv/src/taskpool.cpp
 * @Copyright 2015-2020 Sloong.com. All Rights Reserved
 * @Description: 
 */
/*** 
 * @......................................&&.........................
 * @....................................&&&..........................
 * @.................................&&&&............................
 * @...............................&&&&..............................
 * @.............................&&&&&&..............................
 * @...........................&&&&&&....&&&..&&&&&&&&&&&&&&&........
 * @..................&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&..............
 * @................&...&&&&&&&&&&&&&&&&&&&&&&&&&&&&.................
 * @.......................&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&.........
 * @...................&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&...............
 * @..................&&&   &&&&&&&&&&&&&&&&&&&&&&&&&&&&&............
 * @...............&&&&&@  &&&&&&&&&&..&&&&&&&&&&&&&&&&&&&...........
 * @..............&&&&&&&&&&&&&&&.&&....&&&&&&&&&&&&&..&&&&&.........
 * @..........&&&&&&&&&&&&&&&&&&...&.....&&&&&&&&&&&&&...&&&&........
 * @........&&&&&&&&&&&&&&&&&&&.........&&&&&&&&&&&&&&&....&&&.......
 * @.......&&&&&&&&.....................&&&&&&&&&&&&&&&&.....&&......
 * @........&&&&&.....................&&&&&&&&&&&&&&&&&&.............
 * @..........&...................&&&&&&&&&&&&&&&&&&&&&&&............
 * @................&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&............
 * @..................&&&&&&&&&&&&&&&&&&&&&&&&&&&&..&&&&&............
 * @..............&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&....&&&&&............
 * @...........&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&......&&&&............
 * @.........&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&.........&&&&............
 * @.......&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&...........&&&&............
 * @......&&&&&&&&&&&&&&&&&&&...&&&&&&...............&&&.............
 * @.....&&&&&&&&&&&&&&&&............................&&..............
 * @....&&&&&&&&&&&&&&&.................&&...........................
 * @...&&&&&&&&&&&&&&&.....................&&&&......................
 * @...&&&&&&&&&&.&&&........................&&&&&...................
 * @..&&&&&&&&&&&..&&..........................&&&&&&&...............
 * @..&&&&&&&&&&&&...&............&&&.....&&&&...&&&&&&&.............
 * @..&&&&&&&&&&&&&.................&&&.....&&&&&&&&&&&&&&...........
 * @..&&&&&&&&&&&&&&&&..............&&&&&&&&&&&&&&&&&&&&&&&&.........
 * @..&&.&&&&&&&&&&&&&&&&&.........&&&&&&&&&&&&&&&&&&&&&&&&&&&.......
 * @...&&..&&&&&&&&&&&&.........&&&&&&&&&&&&&&&&...&&&&&&&&&&&&......
 * @....&..&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&...........&&&&&&&&.....
 * @.......&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&..............&&&&&&&....
 * @.......&&&&&.&&&&&&&&&&&&&&&&&&..&&&&&&&&...&..........&&&&&&....
 * @........&&&.....&&&&&&&&&&&&&.....&&&&&&&&&&...........&..&&&&...
 * @.......&&&........&&&.&&&&&&&&&.....&&&&&.................&&&&...
 * @.......&&&...............&&&&&&&.......&&&&&&&&............&&&...
 * @........&&...................&&&&&&.........................&&&..
 * @.........&.....................&&&&........................&&....
 * @...............................&&&.......................&&......
 * @................................&&......................&&.......
 * @.................................&&..............................
 * @..................................&..............................
 */

#include "stdafx.h"
#include "univ.h"
#include "taskpool.h"
#include "threadpool.h"

#ifndef _WINDOWS
#include <unistd.h>
#endif

using namespace Sloong;
using namespace Sloong::Universal;

class TaskInner : public Task
{
public:
	virtual void Wait()
	{
		if (state == TaskState::Created || state == TaskState::Running)
		{
			sync.wait();
		}
	}

	virtual TaskState Status() const
	{
		return state;
	}

public:
	void Call()
	{
		state = TaskState::Running;
		try
		{
			CallInner();
			state = TaskState::RanToCompletion;
		}
		catch (exception &e)
		{
			state = TaskState::Faulted;
		}
		sync.notify_all();
	}

protected:
	virtual void CallInner() = 0;

public:
	TaskPriority priority;
	EasySync sync;
	TaskState state = TaskState::Created;
};

class Task_C : public TaskInner
{
protected:
	void CallInner()
	{
		if (pJob != nullptr)
		{
			auto pRes = (*pJob)(pParam);
			if (pCallback != nullptr)
				(*pCallback)(pRes);
		}
	}

public:
	LPTASKFUNC pJob = nullptr;
	LPTASKCALLBACK pCallback = nullptr;
	LPVOID pParam = nullptr;
};

class Task_CPP : public TaskInner
{
protected:
	void CallInner()
	{
		if (pJobFunc != nullptr)
		{
			auto pRes = pJobFunc();
			if (pCallback != nullptr)
				pCallback(pRes);
		}
	}

public:
	std::function<shared_ptr<void>()> pJobFunc;
	std::function<void(shared_ptr<void>)> pCallback;
};

typedef priority_queue<shared_ptr<TaskInner>, vector<shared_ptr<TaskInner>>, std::function<bool(shared_ptr<TaskInner>, shared_ptr<TaskInner>)>> priority_task_queue;

class TasksInner
{
public:
	TasksInner()
	{
		m_oJobList = make_unique<priority_task_queue>([](shared_ptr<TaskInner> left, shared_ptr<TaskInner> right) { return left->priority < right->priority; });
	}

	shared_ptr<Task> Run(LPTASKFUNC pJob, LPVOID pParam, LPTASKCALLBACK pCallback, TaskPriority priority)
	{
		auto pItem = make_shared<Task_C>();
		pItem->pJob = pJob;
		pItem->pParam = pParam;
		pItem->pCallback = pCallback;
		pItem->priority = priority;
		std::lock_guard<mutex> lck(g_oJobListMutex);
		m_oJobList->push(pItem);
		g_oSync.notify_one();
		return pItem;
	}

	shared_ptr<Task> Run(std::function<shared_ptr<void>()> pJob, std::function<void(shared_ptr<void>)> pCallback, TaskPriority priority)
	{
		auto pItem = make_shared<Task_CPP>();
		pItem->pJobFunc = pJob;
		pItem->pCallback = pCallback;
		pItem->priority = priority;
		std::lock_guard<mutex> lck(g_oJobListMutex);
		m_oJobList->push(pItem);
		g_oSync.notify_one();
		return pItem;
	}

	void ThreadWorkLoop(void)
	{
		while (!g_bExit)
		{
			try
			{
				// each 1 second check the list once.
				if (m_oJobList->empty())
				{
					g_oSync.wait_for(1000);
					continue;
				}

				// Run the job list first.
				if (!m_oJobList->empty())
				{
					unique_lock<mutex> list_lock(g_oJobListMutex);
					if (m_oJobList->empty())
					{
						list_lock.unlock();
						continue;
					}

					auto pJobInfo = m_oJobList->top();
					m_oJobList->pop();
					list_lock.unlock();
					pJobInfo->Call();
				}
			}
			catch (...)
			{
				printf("Unknown error happened in threadpool work loop.");
			}
		}
		cout << "Task pool work thread exit.";
	}

	void ClearAll()
	{
		unique_lock<mutex> list_lock(g_oJobListMutex);
		while (!m_oJobList->empty())
			m_oJobList->pop();
	}

	void Exit()
	{
		ClearAll();
		g_bExit = true;
		g_oSync.notify_all();
	}

protected:
	// list<shared_ptr<Task>> m_oJobList;
	unique_ptr<priority_task_queue> m_oJobList = nullptr;
	mutex g_oJobListMutex;
	bool g_bExit = false;
	EasySync g_oSync;
};

unique_ptr<TasksInner> GLOBAL_TASKS_INSTANCE = make_unique<TasksInner>();

void Sloong::Universal::TaskPool::Initialize(int nThreadNum)
{
	ThreadPool::AddWorkThread(std::bind(&TasksInner::ThreadWorkLoop, GLOBAL_TASKS_INSTANCE.get()), nThreadNum);
}

void Sloong::Universal::TaskPool::ClearAll()
{
	GLOBAL_TASKS_INSTANCE->ClearAll();
}

void Sloong::Universal::TaskPool::Exit()
{
	GLOBAL_TASKS_INSTANCE->Exit();
}

shared_ptr<Task> Sloong::Universal::TaskPool::Run(LPTASKFUNC pJob, LPVOID pParam, LPTASKCALLBACK pCallback, TaskPriority priority)
{
	return GLOBAL_TASKS_INSTANCE->Run(pJob, pParam, pCallback, priority);
}

shared_ptr<Task> Sloong::Universal::TaskPool::Run(std::function<shared_ptr<void>()> pJob, std::function<void(shared_ptr<void>)> pCallback,  TaskPriority priority)
{
	return GLOBAL_TASKS_INSTANCE->Run(pJob, pCallback, priority);
}
